<?php

namespace Drupal\social_links;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

class SocialLink {

  /**
   * @var string
   */
  protected $name;

  /**
   * @var array
   */
  protected $config;

  public function __construct(string $name, array $config) {
    if (!array_key_exists('callback', $config) || !array_key_exists('path', $config)) {
      throw new \Exception('Social Links: "' . $name . '" callback AND path are required.');
    }

    $this->name = $name;
    $this->config = $config;
  }

  /**
   * Render link for a given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to render the links for.
   *
   * @return \Drupal\Core\Link
   *   Link object.
   */
  public function render(EntityInterface $entity) {
    $linkOptions = $this->buildLinkOptions();

    if ($this->hasValidCallback()) {
      $this->config['callback']($linkOptions, $entity);
    }

    $url = Url::fromUri($linkOptions['path']);
    $url->setOptions($linkOptions);
    $link = Link::fromTextAndUrl($this->getLinkTitle(), $url);

    return $link;
  }

  /**
   * Get the provider name.
   *
   * @return string
   *   The provider name.
   */
  public function getName() {
    return $this->name;
  }

  /**
   * Get link title.
   *
   * If SVG is set, render it and add it in place of the title.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup|mixed
   *   Translatable title or SVG.
   */
  protected function getLinkTitle() {
    if ($svg = $this->getConfigValue('svg')) {
      $svg = [
        '#markup' => Markup::create(
          '<svg class="icon"><use xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="#' . $svg . '"></use></svg>'
        )
      ];
      return render($svg);
    }

    return t(ucfirst($this->getName()));
  }

  /**
   * Build link options.
   *
   * @return array
   *   Link options array.
   */
  protected function buildLinkOptions() {
    $request = \Drupal::request();
    $routeMatch = \Drupal::routeMatch();
    $pageTitle = urlencode(
      \Drupal::service('title_resolver')->getTitle($request, $routeMatch->getRouteObject())
    );
    $entityUrl = urlencode($request->getUri());

    $provider = $this->getName();
    $link_class = $provider . '-social-link';
    return [
      'path' => $this->getConfigValue('path') . $entityUrl . '&amp;title=' . $pageTitle,
      'html' => true,
      'attributes' => [
        'title' => t(ucfirst($provider)),
        'class' => [
          'social-link',
          $link_class,
        ],
      ],
    ];
  }

  /**
   * Check if the callback provided is valid.
   *
   * @return bool
   *   If the callback is callable.
   */
  public function hasValidCallback() {
    if (is_callable($this->getConfigValue('callback'))) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Get config value from array key.
   *
   * @param $key
   *   The key of the config.
   *
   * @return string|bool
   *   The associated value or false.
   */
  public function getConfigValue($key) {
    if (array_key_exists($key, $this->config) && !empty($this->config[$key])) {
      return $this->config[$key];
    }

    return FALSE;
  }
}
